//
//  mmDynamicTypeExtensions.swift
//  iRateFlyer
//
//  Created by Kevin McNeish on 1/27/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import Foundation
import UIKit

// Dynamic Type change handler
// Adopt this on controls to adapt to Dynamic Type changes
protocol DynamicTypeChangeHandler {
    
    var typeObserver: Bool {get set}
}

extension UILabel : DynamicTypeChangeHandler {
    
    @IBInspectable var typeObserver: Bool {
        
        get {
            return DynamicTypeManager.sharedInstance.isControlRegistered(self)
        }
        
        set {
            DynamicTypeManager.sharedInstance.registerControl(self, fontKeyPath: "font")
        }
    }
}

//extension UIButton : DynamicTypeChangeHandler {
//
//    @IBInspectable var typeObserver: Bool {
//
//        get {
//            return DynamicTypeManager.sharedInstance.isControlRegistered(self)
//        }
//
//        set {
//
//            DynamicTypeManager.sharedInstance.registerControl(self, fontKeyPath: "titleLabel?.font")
//        }
//    }
//
//}

//extension UITextField : DynamicTypeChangeHandler {
//
//    @IBInspectable var typeObserver: Bool {
//
//        get {
//            return DynamicTypeManager.sharedInstance.isControlRegistered(self)
//        }
//
//        set {
//            DynamicTypeManager.sharedInstance.registerControl(self, fontKeyPath: "font")
//        }
//    }
//}

//extension UITextView : DynamicTypeChangeHandler {
//
//    @IBInspectable var typeObserver: Bool {
//
//        get {
//            return DynamicTypeManager.sharedInstance.isControlRegistered(self)
//        }
//
//        set {
//            DynamicTypeManager.sharedInstance.registerControl(self, fontKeyPath: "font")
//            // Register the placeholder label
//            if self.placeholderLabel != nil {
//                            DynamicTypeManager.sharedInstance.registerControl(self.placeholderLabel!, fontKeyPath: "font")
//            }
//        }
//    }
//}

extension UISegmentedControl: DynamicTypeChangeHandler {
    
    @IBInspectable var typeObserver: Bool {
        
        get {
            return DynamicTypeManager.sharedInstance.isControlRegistered(self)
        }
        
        set {
            
            var attributes = self.titleTextAttributesForState(UIControlState.Normal)
            DynamicTypeManager.sharedInstance.registerControl(self, font: attributes[NSFontAttributeName] as! UIFont) { (fontStyle) -> () in
                
                var attributes = [NSFontAttributeName: UIFont.preferredFontForTextStyle(fontStyle)]
                self.setTitleTextAttributes(attributes, forState: UIControlState.Normal)
            }
        }
    }
}


// Dynamic Type Text Styles
let textStyles = [UIFontTextStyleBody,
    UIFontTextStyleCaption1,
    UIFontTextStyleCaption2,
    UIFontTextStyleFootnote,
    UIFontTextStyleHeadline,
    UIFontTextStyleSubheadline]

// Contains the font key path and style
class DynamicTypeControlValues: NSObject {
    var fontKeyPath: String?
    var fontStyle: String
    var closure: ((fontStyle: String) -> ())?
    init(fontKeyPath: String, fontStyle: String) {
        self.fontKeyPath = fontKeyPath
        self.fontStyle = fontStyle
    }
    init(style: String, closure:(fontStyle: String) -> ()) {
        self.fontStyle = style
        self.closure = closure
    }
}

// Implement SequenceType protocol on the NSMapTable
// so you can use for-in to iterate it in Swift
extension NSMapTable: SequenceType {
    public func generate() -> NSFastGenerator {
        return NSFastGenerator(self)
    }
}

// Dynamic Type Manager (Singleton)
private let dynamicTypeManager = DynamicTypeManager()
class DynamicTypeManager {
    
    // Reference to the DynamicTypeManager shared instance
    class var sharedInstance: DynamicTypeManager {
        return dynamicTypeManager
    }
    
    // Map table holds a weak reference to registered controls
    var registeredElements: NSMapTable
    
    init () {
        
        self.registeredElements = NSMapTable.weakToStrongObjectsMapTable()
        
        // Register the Dynamic Type Manager for Dynamic Type Change notifications
        NSNotificationCenter.defaultCenter().addObserverForName(UIContentSizeCategoryDidChangeNotification, object: nil, queue: nil) { (note) -> Void in
            
            // This block responds to the Dynamic Type Change notification
            var registeredElements = self.registeredElements
            
            for (element) in registeredElements {
                var control = element as! UIView
                
                // Get this control's font key path and style
                var controlValues = registeredElements.objectForKey(control) as! DynamicTypeControlValues
                
                if controlValues.closure == nil {
                    // Set the new font for the specified style
                    control.setValue(UIFont.preferredFontForTextStyle(controlValues.fontStyle), forKeyPath: controlValues.fontKeyPath!)
                    
                }
                else {
                    controlValues.closure!(fontStyle: controlValues.fontStyle)
                }
                // Resize the control to fit
                control.sizeToFit()
                
            }
        }
    }
    
    // Register a UI control
    func registerControl(control: UIView, fontKeyPath: String) {
        
        // Get the control's current font and derive it's style
        var font = control.valueForKeyPath(fontKeyPath) as! UIFont
        var fontStyle = self.fontStyleForFont(font)
        
        
        // Register the control along with its font keypath and font style
        self.registeredElements.setObject(DynamicTypeControlValues(fontKeyPath: fontKeyPath, fontStyle: fontStyle as String), forKey: control)
    }
    
    // Register a UI control with a closure for callback
    func registerControl(control: UIView, font: UIFont, withClosure:(fontStyle: String) -> ()){
        
        var style = fontStyleForFont(font)
        
        // Register the control along with its font style
        self.registeredElements.setObject(DynamicTypeControlValues(style: style as String, closure: withClosure), forKey: control)
        
    }
    
    // Returns true if the specified control is registered
    func isControlRegistered(control: UIView) -> Bool {
        
        return self.registeredElements.objectForKey(control) != nil
    }
    
    // Returns the font style for the specified font
    func fontStyleForFont(font: UIFont) -> NSString {
        
        var fontStyle: String!
        
        for style in textStyles {
            if font.isEqual(UIFont.preferredFontForTextStyle(style)) {
                fontStyle = style
                break
            }
        }
        
        assert(fontStyle != nil, "You must specify a style rather than a specific font type for this control.")
        
        return fontStyle
    }
    
    deinit {
        // Remove the Dynamic Type Manager from notifications
        NSNotificationCenter.defaultCenter().removeObserver(self)
    }
}





