//
//  mmStarRating.swift
//  mmiOS8
//
//  Created by Kevin McNeish on 1/7/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit

protocol StarRatingDelegate {
    
    func starRating(starRating: mmStarRating, ratingDidChange: Float)
}


class mmStarRating: UIView,  DynamicTypeChangeHandler {
    
    
    @IBInspectable var typeObserver: Bool {
        
        get {
            return DynamicTypeManager.sharedInstance.isControlRegistered(self)
        }
        
        set {

            // Register the placeholder label
            if self.displayRatingText {
                DynamicTypeManager.sharedInstance.registerControl(self.ratingLabel!, fontKeyPath: "font")
            }
        }
    }
    
    var unselectedImage = UIImage(named: "star")
    var halfSelectedImage = UIImage(named: "halfstar")
    var selectedImage = UIImage(named: "fullstar")
    var ratingText = ["Terrible", "Poor", "Average", "Very Good", "Excellent"]
    var editable = true
    var imageViews = Array<UIImageView>()
    
    var midMargin = 5
    var leftMargin = 0
    var labelMargin = 5
    var minImageSize = CGSizeMake(5, 5)
    var delegate: StarRatingDelegate?
    var ratingLabel: UILabel?
    var displayRatingText = true
    var rating: Double = 0.0 {
        didSet {
            self.refresh()
            self.setRatingText()
        }
    }
    var maxRating: Int = 5 {
        didSet {
            self.createImageViews()
        }
    }
    
    required init(coder aDecoder: NSCoder) {
        
        super.init(coder: aDecoder)
        
        self.baseInit()
    }
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        self.baseInit()
    }
    
    func baseInit() {
        self.createImageViews()
    }
    
    func createImageViews() {
        
        // Remove old image views
        for i in 0..<self.imageViews.count {
            let imageView = self.imageViews[i]
            imageView.removeFromSuperview()
        }
        self.imageViews.removeAll(keepCapacity: true)
        if let ratingLabel = self.ratingLabel {
            ratingLabel.removeFromSuperview()
        }
        
        // Add new images
        for i in 0..<self.maxRating {
            var imageView = UIImageView()
            imageView.contentMode = UIViewContentMode.ScaleAspectFit
            self.imageViews.append(imageView)
            self.addSubview(imageView)
        }
        
        if self.displayRatingText {
            self.ratingLabel = UILabel()
            self.ratingLabel?.font = UIFont.preferredFontForTextStyle(UIFontTextStyleSubheadline)
            self.addSubview(self.ratingLabel!)
        }
        
        // Re-layout and refresh
        self.setNeedsLayout()
        self.refresh()
    }
    
    func refresh() {
        
        for i in 0..<self.imageViews.count {
            var imageView = self.imageViews[i]
            if self.rating >= Double(i)+1.0 {
                imageView.image = self.selectedImage
            }
            else if (self.rating > Double(i)) {
                imageView.image = self.halfSelectedImage
            }
            else {
                imageView.image = self.unselectedImage
            }
        }
    }
    
    override func layoutSubviews() {
        super.layoutSubviews()
        
        if (self.unselectedImage != nil) {
            
            var x: CGFloat = CGFloat(1)
            
            var desiredImageWidth = (self.frame.size.width -
                CGFloat(self.leftMargin * 2) -
                CGFloat(self.midMargin * self.imageViews.count)) /
                CGFloat(self.imageViews.count)
            
            var imageWidth = max(self.minImageSize.width, desiredImageWidth)
            var imageHeight = max(self.minImageSize.height, self.frame.size.height - CGFloat(self.labelMargin) - CGFloat(21))
            
            var imageView: UIImageView!
            for i in 0..<self.imageViews.count {
                imageView = self.imageViews[i]
                let imageFrame = CGRectMake(CGFloat(self.leftMargin) + CGFloat(i)*((CGFloat(self.midMargin) + imageWidth)), CGFloat(0), imageWidth, imageHeight)
                imageView.frame = imageFrame
            }
            
            if self.displayRatingText {
                
                let labelFrame = CGRectMake(CGFloat(self.leftMargin), imageView.frame.origin.y + CGFloat(self.labelMargin) + imageView.frame.size.height, self.frame.size.width, 21)
                self.ratingLabel!.frame = labelFrame
                self.ratingLabel!.backgroundColor = UIColor.clearColor()
                self.ratingLabel!.textColor = UIColor.blackColor()
                self.ratingLabel!.textAlignment = NSTextAlignment.Center
                self.hidden = false
                
                self.setRatingText()
            }
        }
    }
    
    func handleTouchAtLocation(touchLocation:CGPoint) {
        
        if !self.editable {
            return
        }
        var newRating = 1	// Set the minimum
        
        for var i = (self.imageViews.count - 1); i >= 0; i-- {
            
            var imageView = self.imageViews[i]
            if touchLocation.x > imageView.frame.origin.x {
                newRating = i+1
                break
            }
        }
        self.rating = Double(newRating)
    }
    
    override func touchesBegan(touches: Set<NSObject>, withEvent event: UIEvent) {
        
        var touch = touches.first as! UITouch
        var touchLocation = touch.locationInView(self)
        self.handleTouchAtLocation(touchLocation)
    }
    
    override func touchesMoved(touches: Set<NSObject>, withEvent event: UIEvent) {
        var touch = touches.first as! UITouch
        var touchLocation = touch.locationInView(self)
        self.handleTouchAtLocation(touchLocation)
    }
    
    override func touchesEnded(touches: Set<NSObject>, withEvent event: UIEvent) {
        
        if let delegate = self.delegate {
            delegate.starRating(self, ratingDidChange: Float(self.rating))
        }
    }
    
    func setRatingText() {
        
        if let ratingLabel = self.ratingLabel {
            
            if self.rating == 0 {
                ratingLabel.text = "Tap to Rate"
            }
            else {
                var roundingRate = Int(roundf(Float(self.rating)))
                roundingRate--
                
                ratingLabel.text = self.ratingText[roundingRate]
            }
        }
    }
    
}

